'use strict';

var _ = require('lodash');
var AwaitLock = require('await-lock');
var KindaObject = require('kinda-object');

var KindaWebSQL = KindaObject.extend('KindaWebSQL', function () {
  this.creator = function () {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    if (!options.name) throw new Error('WebSQL database name is missing');
    var shortName = options.name;
    var version = options.version || '1.0';
    var displayName = options.displayName || options.name;
    var maxSize = options.maxSize || 50 * 1024 * 1024; // 50 MB
    this.database = openDatabase(shortName, version, displayName, maxSize);
    this.awaitLock = new AwaitLock();
  };

  this.lock = function callee$1$0(fn) {
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          context$2$0.next = 2;
          return regeneratorRuntime.awrap(this.awaitLock.acquireAsync());

        case 2:
          context$2$0.prev = 2;
          context$2$0.next = 5;
          return regeneratorRuntime.awrap(fn());

        case 5:
          return context$2$0.abrupt('return', context$2$0.sent);

        case 6:
          context$2$0.prev = 6;

          this.awaitLock.release();
          return context$2$0.finish(6);

        case 9:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this, [[2,, 6, 9]]);
  };

  this.query = function callee$1$0(sql, values) {
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          context$2$0.next = 2;
          return regeneratorRuntime.awrap(this.lock((function callee$2$0() {
            return regeneratorRuntime.async(function callee$2$0$(context$3$0) {
              while (1) switch (context$3$0.prev = context$3$0.next) {
                case 0:
                  context$3$0.next = 2;
                  return regeneratorRuntime.awrap(this._query(sql, values));

                case 2:
                  return context$3$0.abrupt('return', context$3$0.sent);

                case 3:
                case 'end':
                  return context$3$0.stop();
              }
            }, null, this);
          }).bind(this)));

        case 2:
          return context$2$0.abrupt('return', context$2$0.sent);

        case 3:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };

  this._query = function callee$1$0(sql, values) {
    var result;
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          values = this.normalizeValues(values);
          context$2$0.next = 3;
          return regeneratorRuntime.awrap(this.__query(sql, values));

        case 3:
          result = context$2$0.sent;

          result = this.normalizeResult(result);
          return context$2$0.abrupt('return', result);

        case 6:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };

  this.__query = function (sql, values) {
    var _this = this;

    return new Promise(function (resolve, reject) {
      var result = undefined;
      _this.database.transaction(function (tr) {
        tr.executeSql(sql, values, function (innerTr, res) {
          result = res;
        });
      }, function (err) {
        // transaction error callback
        reject(err);
      }, function () {
        // transaction success callback
        resolve(result);
      });
    });
  };

  this.transaction = function callee$1$0(fn) {
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          context$2$0.next = 2;
          return regeneratorRuntime.awrap(this.lock((function callee$2$0() {
            return regeneratorRuntime.async(function callee$2$0$(context$3$0) {
              while (1) switch (context$3$0.prev = context$3$0.next) {
                case 0:
                  context$3$0.next = 2;
                  return regeneratorRuntime.awrap(fn({ query: this._query.bind(this) }));

                case 2:
                  return context$3$0.abrupt('return', context$3$0.sent);

                case 3:
                case 'end':
                  return context$3$0.stop();
              }
            }, null, this);
          }).bind(this)));

        case 2:
          return context$2$0.abrupt('return', context$2$0.sent);

        case 3:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };

  this.normalizeValues = function (values) {
    if (values && values.length) {
      values = _.map(values, function (val) {
        if (typeof val === 'undefined') {
          val = null;
        } else if (Buffer.isBuffer(val)) {
          val = 'bin!' + val.toString('hex');
        }
        return val;
      });
    }
    return values;
  };

  this.normalizeResult = function (result) {
    if (!result) return result;
    var normalizedResult = [];
    if (result.rowsAffected != null) {
      normalizedResult.affectedRows = result.rowsAffected;
    }
    try {
      if (result.insertId != null) {
        normalizedResult.insertId = result.insertId;
      }
    } catch (err) {
      // noop
    }
    if (!result.rows) return normalizedResult;

    var _loop = function (i) {
      var row = result.rows.item(i);
      var normalizedRow = {};
      _.forOwn(row, function (val, key) {
        // eslint-disable-line no-loop-func
        if (val && val.substr && val.substr(0, 4) === 'bin!') {
          val = new Buffer(val.substr(4), 'hex');
        }
        normalizedRow[key] = val;
      });
      normalizedResult.push(normalizedRow);
    };

    for (var i = 0; i < result.rows.length; i++) {
      _loop(i);
    }
    return normalizedResult;
  };
});

module.exports = KindaWebSQL;